#!/bin/bash

### The simulation part of the neural network. This is run by individual chunks in parallel with each other

### 1. Creates directories for each member if they havent already done so, i.e. in the first generation
### 2. Edits the namelists of each member using the corresponding row of the 00.csv parameters file
### 3. Runs the simulation
### 4. Uses the outputs, reference bias and observed data to collect the cost and save as a .txt

## ALL SHOULD BE THE SAME AS THOSE IN pre.sh
SCRHOME=/gpfs/scratch/bsc32/bsc32786/
MODEL=${SCRHOME}%EXPID%/EXP00/
OBS=${SCRHOME}pisces1d/ORCA2_OFF_PISCES/obs/bbp700_pisces_climatology_lab1d.nc
MODELFILE=PISCES1D_1d_19600101_19691231_ptrc_T.nc


### 1. Creates directories for each member if they havent already done so, i.e. in the first generation

shopt -s nullglob
numfiles=(${MODEL}parameters/*)
numfiles=${#numfiles[@]}

INPUTS=input/
XIOS=xios-2.5/
NEMO=release-4.0/

chunk=%CHUNK%
printf -v chunk "%02d" $chunk

if [ $numfiles -eq $((1)) ]; then
    # MAKE EXPID DIRECTORY AND COPY ALL THE CONTENT FROM EXP00 (.XML, EXECUTABLES, AND NAMELISTS)
    cd ${MODEL}
    mkdir -p $chunk
    cd $chunk
    cp -R ${MODEL}default/nemo .
    cp -R ${MODEL}default/namelist* .
    cp -R ${MODEL}default/*.xml .
    cp -R ${MODEL}cropped/* .
fi


### 2. Edits the namelists of each member using the corresponding row of the 00.csv parameters file

VARIABLES=variables.csv
NAMELIST=namelist_pisces_ref

cd ${MODEL}$chunk/

module load python

function changenls {

nls="$(python - $1 $2 $3 $4 << END
# inputs:
# 1. Parameters file (00.csv)
# 2. Variables file (variables.csv)
# 3. Namelist (namelist_pisces_ref)
# 4. Chunk number

import os
import pandas as pd
import numpy as np

values=np.genfromtxt('$1')
names=pd.read_csv('$2')

chunk=int('$4')-1

with open('$3','r') as nl:
    with open('temp','w+') as t:
        lines=nl.readlines()
        for l in lines:
            for j in range(len(values[chunk])):
                if names['var'][j]+' ' in l and "=" in l:
                    #t.write('   '+ names['var'][j] + '  =  ' + str(values[chunk][j]))
                    l = '   '+ names['var'][j] + '  =  ' + str(values[chunk][j]) + '\n'
                    #print(new)
                #else:
                    #t.write(l)
                    #new = l
            t.write(l)

os.remove('$3')
os.rename('temp','$3')

END
)"
}

changenls ${MODEL}/parameters/00.csv ${MODEL}$VARIABLES ${MODEL}$chunk/$NAMELIST %CHUNK%


### 3. Runs the simulation

cd ${MODEL}$chunk

module load intel/2017.4
module load impi/2017.4
module load netcdf/4.2
module load hdf5/1.8.19
module load perl

mpirun -np 1 ./nemo

### 4. Uses the outputs, reference bias and observed data to collect the cost and save as a .txt

module load netcdf/4.4.1.1

function getcost {

cost="$(python - $1 $2 $3 << END
# $1 - path to reference bias, $2 - path to observed data $3 - Path to output

from netCDF4 import Dataset
import numpy as np
import os
import math

# get the reference bias
os.chdir('$1')
f=open('refbias.txt')
refbias=f.readlines()
refbias=float(refbias[0])
f.close()

# get the observed data
#os.chdir('$2')
#obs=Dataset('bbp700_pisces_climatology_lab1d.nc')
obs=Dataset('$2')
bbp700obs = np.array(obs.variables['bbp700_all_mean'])
bbp700obs = np.reshape(bbp700obs, (np.shape(bbp700obs)[0], np.shape(bbp700obs)[1]))
bbp700obs[bbp700obs == 0.0] = np.nan
obs.close()

# get the model data
os.chdir('$3')
mod = Dataset('PISCES1D_1d_19600101_19691231_ptrc_T.nc')
bbp700mod = 2500*(np.array(mod.variables['POC'])[-60:, :, 1, 1]+np.array(mod.variables['PHY'])[-60:, :, 1, 1])
for i in range(12):
    bbp700mod[i] = np.mean(bbp700mod[i::12],0)
bbp700mod = bbp700mod[:12]
bbp700mod[bbp700mod == 200000039192547557376] = np.nan
mod.close()

# converting the data into logspace and normalising based on observed data
bbp700obs=np.log(bbp700obs)
bbp700mod=np.log(bbp700mod)

std=np.nanstd(bbp700obs)
mn=np.nanmean(bbp700obs)

bbp700obs=(bbp700obs-mn)/std
bbp700mod=(bbp700mod-mn)/std

# calculating the cost
bias=np.nanmean(bbp700obs-bbp700mod)
mbias=bias/refbias
mstd=np.nanstd(bbp700mod)
R=float(np.ma.corrcoef(np.ma.masked_invalid(bbp700obs.flatten()), np.ma.masked_invalid(bbp700mod.flatten()))[0,1])
S3=1.-(math.exp(-((mstd-1.)**2)/0.18))*((1+R)/2)

ST=math.sqrt((mbias**2)+(S3**2))

print(ST)

END
)"
}

getcost ${MODEL}default/ $OBS $MODEL$chunk

# returns the ST value (cost function) and saves as text file
cost=(${cost})

echo $cost

cd $MODEL$chunk
touch cost.txt
echo $cost > cost.txt